@@ -596,6 +596,9 @@ class DMTEVT_model(LightningModule):
         features_input_a = input_data[:batch_size]
         features_input_b = input_data[batch_size:]
         dis_input_ab = self._DistanceSquared(features_input_a, features_input_b) * temperature
+        diag_dis_input_ab = torch.diagonal(dis_input_ab).detach()
+        dis_input_ab = dis_input_ab - diag_dis_input_ab.unsqueeze(1)/2 - diag_dis_input_ab.unsqueeze(0)/2
+        
         P = self.UMAPNoSigmaSimilarity(dis_input_ab, nu=100)
         P[torch.eye(P.shape[0]) == 1] = 1.0
 
@@ -604,7 +607,42 @@ class DMTEVT_model(LightningModule):
         losssum2 = (1 - P) * torch.log(1 - Q + EPS)
         loss = -1 * (losssum1 + losssum2).mean()
         
-        print('loss:', loss.item())
+        # print('loss:', loss.item())
+
+        return loss
+    
+    def LossManifold_All(self, input_data, latent_data, temperature=1, exaggeration=1, nu=0.1):
+        """
+        Computes the manifold loss between two views of the data.
+
+        Args:
+            latent_data (Tensor): Latent representations of shape (2 * batch_size, ...).
+            temperature (float): Temperature scaling.
+            exaggeration (float): Exaggeration factor.
+            nu (float): Degrees of freedom for t-distribution.
+
+        Returns:
+            loss (Tensor): Computed loss.
+        """
+        loss1 = self.LossManifold_Global(
+            input_data=input_data,
+            latent_data=latent_data,
+            temperature=temperature,
+            exaggeration=exaggeration,
+            nu=nu
+        ) 
+        
+        loss1 = loss1/loss1.detach()
+        
+        loss2 = self.LossManifold(
+            input_data=input_data,
+            latent_data=latent_data,
+            temperature=temperature,
+            exaggeration=exaggeration,
+            nu=nu
+        ) 
+        loss2 = loss2/loss2.detach()
+        loss = loss1 * 0.5 + loss2 * 0.5
 
         return loss
 
@@ -780,6 +818,8 @@ class DMTEVT_model(LightningModule):
             LossFunc = self.LossManifold_Global
         elif self.hparams.loss_type == 'L':
             LossFunc = self.LossManifold
+        elif self.hparams.loss_type == 'A':
+            LossFunc = self.LossManifold_All
         
         # Compute manifold losses
         loss_emb = LossFunc(
